#ifndef PCS_STATE_CPP
#define PCS_STATE_CPP

//---------------------------------------------------------------------
//    ___
//   /\__\    This file is part of MUSE    <http://www.muse-tools.org/>
//  /::L_L_
// /:/L:\__\  Miami   University  Simulation  Environment    (MUSE)  is
// \/_/:/  /  free software: you can  redistribute it and/or  modify it
//   /:/  /   under the terms of the GNU  General Public License  (GPL)
//   \/__/    as published  by  the   Free  Software Foundation, either
//            version 3 (GPL v3), or  (at your option) a later version.
//    ___
//   /\__\    MUSE  is distributed in the hope that it will  be useful,
//  /:/ _/_   but   WITHOUT  ANY  WARRANTY;  without  even  the IMPLIED
// /:/_/\__\  WARRANTY of  MERCHANTABILITY  or FITNESS FOR A PARTICULAR
// \:\/:/  /  PURPOSE.
//  \::/  /
//   \/__/    Miami University  and  the MUSE  development team make no
//            representations  or  warranties  about the suitability of
//    ___     the software,  either  express  or implied, including but
//   /\  \    not limited to the implied warranties of merchantability,
//  /::\  \   fitness  for a  particular  purpose, or non-infringement.
// /\:\:\__\  Miami  University and  its affiliates shall not be liable
// \:\:\/__/  for any damages  suffered by the  licensee as a result of
//  \::/  /   using, modifying,  or distributing  this software  or its
//   \/__/    derivatives.
//
//    ___     By using or  copying  this  Software,  Licensee  agree to
//   /\  \    abide  by the intellectual  property laws,  and all other
//  /::\  \   applicable  laws of  the U.S.,  and the terms of the  GNU
// /::\:\__\  General  Public  License  (version 3).  You  should  have
// \:\:\/  /  received a  copy of the  GNU General Public License along
//  \:\/  /   with MUSE.  If not,  you may  download  copies  of GPL V3
//   \/__/    from <http://www.gnu.org/licenses/>.
//
//---------------------------------------------------------------------

#include "PCS_State.h"
#include <sstream>

PCS_State::PCS_State() : index(0), blockedChannels(0),
                         callAttempts(0), handOffBlocks(0),
                         idleChannels(0), busyLineCalls(0),
                         moveIns(0), moveOuts(0) {
    // Nothing else to be done in the constructor.
}

PCS_State::PCS_State(uint idleChannels) :
    index(0), blockedChannels(0), callAttempts(0), handOffBlocks(0),
    idleChannels(idleChannels), busyLineCalls(0), moveIns(0), moveOuts(0) {
    // Nothing else to be done in the constructor.
}

muse::State*
PCS_State::getClone(){
    // Keep in mind this a shallow copy.  However, it works here only
    // because there are no complex pointers/objects in the state but
    // just primitive data types.  So the default copy-constructor
    // generated by the compiler is sufficient.
    return new PCS_State(*this);
}

std::string
PCS_State::to_string() const {
    std::ostringstream os;
    os << "blocked channels: "  << blockedChannels 
       << ", call attempts: "   << callAttempts
       << ", hand-off blocks: " << handOffBlocks
       << ", idle channels: "   << idleChannels
       << ", busy-line calls: " << busyLineCalls
       << ", move-ins: "        << moveIns
       << ", move-outs: "       << moveOuts;
    return os.str();
}

#endif
